@@ -96,7 +96,7 @@ def _chunkiter(hash, startofs):
             yield ''.join(cp().join(sha.encode('hex')))[skipmore:]
 
 
-class _ChunkReader(object):
+class _ChunkReader:
     def __init__(self, hash, isdir, startofs):
         if isdir:
             self.it = _chunkiter(hash, startofs)
@@ -120,7 +120,7 @@ class _ChunkReader(object):
                 self.blob = self.blob[want:]
             if not self.it:
                 break
-        log('next(%d) returned %d\n' % (size, len(out)))
+        debug2('next(%d) returned %d\n' % (size, len(out)))
         self.ofs += len(out)
         return out
 
@@ -161,7 +161,7 @@ class _FileReader(object):
         pass
 
 
-class Node(object):
+class Node:
     """Base class for file representation."""
     def __init__(self, parent, name, mode, hash):
         self.parent = parent
@@ -172,15 +172,17 @@ class Node(object):
         self._subs = None
 
     def __cmp__(a, b):
-        return cmp(a.name or None, b.name or None)
+        return cmp(a and a.name or None, b and b.name or None)
 
     def __iter__(self):
         return iter(self.subs())
 
-    def fullname(self):
+    def fullname(self, stop_at=None):
         """Get this file's full path."""
-        if self.parent:
-            return os.path.join(self.parent.fullname(), self.name)
+        assert(self != stop_at)  # would be the empty string; too weird
+        if self.parent and self.parent != stop_at:
+            return os.path.join(self.parent.fullname(stop_at=stop_at),
+                                self.name)
         else:
             return self.name
 
@@ -220,7 +222,7 @@ class Node(object):
             return self
 
     def _lresolve(self, parts):
-        #log('_lresolve %r in %r\n' % (parts, self.name))
+        #debug2('_lresolve %r in %r\n' % (parts, self.name))
         if not parts:
             return self
         (first, rest) = (parts[0], parts[1:])
@@ -253,7 +255,7 @@ class Node(object):
         parts = re.split(r'/+', path or '.')
         if not parts[-1]:
             parts[-1] = '.'
-        #log('parts: %r %r\n' % (path, parts))
+        #debug2('parts: %r %r\n' % (path, parts))
         return start._lresolve(parts)
 
     def resolve(self, path = ''):
@@ -310,12 +312,12 @@ class File(Node):
     def size(self):
         """Get this file's size."""
         if self._cached_size == None:
-            log('<<<<File.size() is calculating...\n')
+            debug1('<<<<File.size() is calculating (for %r)...\n' % self.name)
             if self.bupmode == git.BUP_CHUNKED:
                 self._cached_size = _total_size(self.hash)
             else:
                 self._cached_size = _chunk_len(self.hash)
-            log('<<<<File.size() done.\n')
+            debug1('<<<<File.size() done.\n')
         return self._cached_size
 
 
