@@ -23,23 +23,24 @@ os.environ['PYTHONPATH'] = libpath + ':' + os.environ.get('PYTHONPATH', '')
 os.environ['BUP_MAIN_EXE'] = os.path.abspath(exe)
 os.environ['BUP_RESOURCE_PATH'] = resourcepath
 
+from bup import helpers
 from bup.helpers import *
 
+# after running 'bup newliner', the tty_width() ioctl won't work anymore
+os.environ['WIDTH'] = str(tty_width())
 
 def usage():
-    log('Usage: bup [-?|--help] [-d=BUP_DIR|--bup-dir=BUP_DIR] COMMAND [ARGS]'
-        + '\n\n')
+    log('Usage: bup [-?|--help] [-d BUP_DIR] [--debug] '
+        '<command> [options...]\n\n')
     common = dict(
         ftp = 'Browse backup sets using an ftp-like client',
         fsck = 'Check backup sets for damage and add redundancy information',
         fuse = 'Mount your backup sets as a filesystem',
         help = 'Print detailed help for the given command',
         index = 'Create or display the index of files to back up',
-        join = 'Retrieve a file backed up using "bup split"',
-        ls = 'Browse the files in your backup sets',
-        midx = 'Index objects to speed up future backups',
+        on = 'Backup a remote machine to the local one',
+        restore = 'Extract files from a backup set',
         save = 'Save files into a backup set (note: run "bup index" first)',
-        split = 'Split a single file into its own backup set',
         web = 'Launch a web server to examine backup sets',
     )
 
@@ -68,8 +69,8 @@ if len(argv) < 2:
 
 # Handle global options.
 try:
-    global_args, subcmd = getopt.getopt(argv[1:], '?Vd:',
-                                        ['help', 'version', 'bup-dir='])
+    global_args, subcmd = getopt.getopt(argv[1:], '?VDd:',
+                                    ['help', 'version', 'debug', 'bup-dir='])
 except getopt.GetoptError, ex:
     log('error: ' + ex.msg + '\n')
     usage()
@@ -78,11 +79,14 @@ help_requested = None
 dest_dir = None
 
 for opt in global_args:
-    if opt[0] == '-?' or opt[0] == '--help':
+    if opt[0] in ['-?', '--help']:
         help_requested = True
-    if opt[0] == '-V' or opt[0] == '--version':
+    elif opt[0] in ['-V', '--version']:
         subcmd = ['version']
-    elif opt[0] == '-d' or opt[0] == '--bup-dir':
+    elif opt[0] in ['-D', '--debug']:
+        helpers.buglvl += 1
+        os.environ['BUP_DEBUG'] = str(helpers.buglvl)
+    elif opt[0] in ['-d', '--bup-dir']:
         dest_dir = opt[1]
     else:
         log('error: unexpected option "%s"\n' % opt[0])
